from xml.etree import ElementTree

from highton.fields.field import Field
from highton.fields.field_constants import FieldConstants


class IntegerField(Field):
    def encode(self):
        """

        :return:
        :rtype: xml.etree.ElementTree.Element
        """
        element = ElementTree.Element(
            self.name,
            attrib={'type': FieldConstants.INTEGER},
        )
        element = self._set_nil(element, lambda value: str(value))
        return element

    def decode(self, element):
        """

        :param element:
        :type element: xml.etree.ElementTree.Element
        :return: the parsed int object
        :rtype: int
        """
        text = element.text
        if text:
            return int(text)
        else:
            return None




from highton.models import HightonModel
from highton.highton_constants import HightonConstants
from highton import fields
from highton import call_mixins


class Task(
    HightonModel,
    call_mixins.CreateCallMixin,
    call_mixins.DetailCallMixin,
    call_mixins.DeleteTagCallMixin,
    call_mixins.UpdateCallMixin,
):
    """

    :ivar id: fields.IntegerField(name=HightonConstants.ID)
    :ivar recording_id: fields.IntegerField(name=HightonConstants.RECORDING_ID)
    :ivar subject_id: fields.IntegerField(name=HightonConstants.SUBJECT_ID)
    :ivar subject_type: fields.StringField(name=HightonConstants.SUBJECT_TYPE)
    :ivar subject_name: fields.StringField(name=HightonConstants.SUBJECT_NAME)
    :ivar category_id: fields.IntegerField(name=HightonConstants.CATEGORY_ID, required=True)
    :ivar body: fields.StringField(name=HightonConstants.BODY, required=True)
    :ivar frame: fields.StringField(name=HightonConstants.FRAME, required=True)
    :ivar due_at: fields.DatetimeField(name=HightonConstants.DUE_AT, required=True)
    :ivar alert_at: fields.DatetimeField(name=HightonConstants.ALERT_AT)
    :ivar created_at: fields.DatetimeField(name=HightonConstants.CREATED_AT)
    :ivar author_id: fields.IntegerField(name=HightonConstants.AUTHOR_ID)
    :ivar updated_at: fields.DatetimeField(name=HightonConstants.UPDATED_AT)
    :ivar public: fields.BooleanField(name=HightonConstants.PUBLIC)
    :ivar recurring_period: fields.StringField(name=HightonConstants.RECURRING_PERIOD)
    :ivar anchor_type: fields.IntegerField(name=HightonConstants.ANCHOR_TYPE)
    :ivar done_at: fields.DatetimeField(name=HightonConstants.DONE_AT)
    :ivar owner_id: fields.IntegerField(name=HightonConstants.OWNER_ID)
    """
    TAG_NAME = HightonConstants.TASK
    ENDPOINT = HightonConstants.TASKS

    def __init__(self, **kwargs):
        self.recording_id = fields.IntegerField(name=HightonConstants.RECORDING_ID)
        self.subject_id = fields.IntegerField(name=HightonConstants.SUBJECT_ID)
        self.subject_type = fields.StringField(name=HightonConstants.SUBJECT_TYPE)
        self.subject_name = fields.StringField(name=HightonConstants.SUBJECT_NAME)
        self.category_id = fields.IntegerField(name=HightonConstants.CATEGORY_ID, required=True)
        self.body = fields.StringField(name=HightonConstants.BODY, required=True)
        self.frame = fields.StringField(name=HightonConstants.FRAME, required=True)
        self.due_at = fields.DatetimeField(name=HightonConstants.DUE_AT, required=True)
        self.alert_at = fields.DatetimeField(name=HightonConstants.ALERT_AT)
        self.created_at = fields.DatetimeField(name=HightonConstants.CREATED_AT)
        self.author_id = fields.IntegerField(name=HightonConstants.AUTHOR_ID)
        self.updated_at = fields.DatetimeField(name=HightonConstants.UPDATED_AT)
        self.public = fields.BooleanField(name=HightonConstants.PUBLIC)
        self.recurring_period = fields.StringField(name=HightonConstants.RECURRING_PERIOD)
        self.anchor_type = fields.IntegerField(name=HightonConstants.ANCHOR_TYPE)
        self.done_at = fields.DatetimeField(name=HightonConstants.DONE_AT)
        self.owner_id = fields.IntegerField(name=HightonConstants.OWNER_ID)

        super().__init__(**kwargs)

    def complete(self):
        """
        Complete current task
        :return:
        :rtype: requests.models.Response
        """
        return self._post_request(
            data='',
            endpoint=self.ENDPOINT + '/' + str(self.id) + '/complete'
        )

    @classmethod
    def list_upcoming(cls):
        """
        Returns a collection of upcoming tasks (tasks that have not yet been completed,
        regardless of whether they’re overdue) for the authenticated user

        :return:
        :rtype: list
        """
        return fields.ListField(name=cls.ENDPOINT, init_class=cls).decode(
            cls.element_from_string(
                cls._get_request(endpoint=cls.ENDPOINT + '/upcoming').text
            )
        )

    @classmethod
    def list_assigned(cls):
        """
        Returns a collection of upcoming tasks (tasks that have not yet been completed,
        regardless of whether they’re overdue) that were created by the authenticated user,
        but assigned to somebody else.

        :return:
        :rtype: list
        """
        return fields.ListField(name=cls.ENDPOINT, init_class=cls).decode(
            cls.element_from_string(
                cls._get_request(endpoint=cls.ENDPOINT + '/assigned').text
            )
        )

    @classmethod
    def list_completed(cls):
        """
        Returns a collection of completed tasks.

        :return:
        :rtype: list
        """
        return fields.ListField(name=cls.ENDPOINT, init_class=cls).decode(
            cls.element_from_string(
                cls._get_request(endpoint=cls.ENDPOINT + '/completed').text
            )
        )

    @classmethod
    def list_today(cls):
        """
        Returns a collection of uncompleted tasks due for the rest of today for the authenticated user.

        :return:
        :rtype: list
        """
        return fields.ListField(name=cls.ENDPOINT, init_class=cls).decode(
            cls.element_from_string(
                cls._get_request(endpoint=cls.ENDPOINT + '/today').text
            )
        )

    @classmethod
    def list_all(cls):
        """
        Returns a collection of all tasks visible to the current user.

        :return:
        :rtype: list
        """
        return fields.ListField(name=cls.ENDPOINT, init_class=cls).decode(
            cls.element_from_string(
                cls._get_request(endpoint=cls.ENDPOINT + '/all').text
            )
        )



from highton.call_mixins import Call
from highton import fields


class ListCommentCallMixin(Call):
    """
    A mixin to get all comments of inherited class
    These could be: notes || emails

    """

    COMMENT_OFFSET = 25

    def list_comments(self, page=0):
        """
        Get the comments of current object

        :param page: the page starting at 0
        :return: the emails
        :rtype: list
        """
        from highton.models.comment import Comment
        params = {'page': int(page) * self.COMMENT_OFFSET}

        return fields.ListField(
            name=self.ENDPOINT,
            init_class=Comment
        ).decode(
            self.element_from_string(
                self._get_request(
                    endpoint=self.ENDPOINT + '/' + str(self.id) + '/' + Comment.ENDPOINT,
                    params=params
                ).text
            )
        )



from highton import (
    fields,
    call_mixins,
)
from highton.highton_constants import HightonConstants
from highton.models.contact import Contact


class Company(
    Contact,
    call_mixins.CreateCallMixin,
    call_mixins.UpdateCallMixin,
    call_mixins.DetailCallMixin,
    call_mixins.ListCallMixin,
    call_mixins.DeleteCallMixin,
    call_mixins.ListNoteCallMixin,
    call_mixins.ListEmailCallMixin,
    call_mixins.ListTagCallMixin,
    call_mixins.CreateTagCallMixin,
    call_mixins.DeleteTagCallMixin,
    call_mixins.ListTaskCallMixin,
    call_mixins.CreateNoteCallMixin,
    call_mixins.SearchCallMixin,
):
    """

    :ivar id: fields.IntegerField(name=HightonConstants.ID)
    :ivar author_id: fields.IntegerField(name=HightonConstants.AUTHOR_ID)
    :ivar background: fields.StringField(name=HightonConstants.BACKGROUND)
    :ivar created_at: fields.DatetimeField(name=HightonConstants.CREATED_AT)
    :ivar group_id: fields.IntegerField(name=HightonConstants.GROUP_ID)
    :ivar owner_id: fields.IntegerField(name=HightonConstants.OWNER_ID)
    :ivar updated_at: fields.DatetimeField(name=HightonConstants.UPDATED_AT)
    :ivar visible_to: fields.StringField(name=HightonConstants.VISIBLE_TO)
    :ivar linkedin_url: fields.StringField(name=HightonConstants.LINKEDIN_URL)
    :ivar avatar_url: fields.StringField(name=HightonConstants.AVATAR_URL)
    :ivar tags: fields.ListField(name=HightonConstants.TAGS, init_class=Tag)
    :ivar contact_data: fields.ObjectField(name=HightonConstants.CONTACT_DATA, init_class=ContactData)
    :ivar subject_datas: fields.ListField(name=HightonConstants.SUBJECT_DATAS, init_class=SubjectData)
    :ivar name: fields.StringField(name=HightonConstants.NAME)
    """
    ENDPOINT = HightonConstants.COMPANIES
    TAG_NAME = HightonConstants.COMPANY
    OFFSET = 500

    def __init__(self, **kwargs):
        self.name = fields.StringField(name=HightonConstants.NAME)

        super().__init__(**kwargs)

    @classmethod
    def list(cls, page=0, since=None, tag_id=None, title=None):
        """

        :param page: page starting by 0
        :type page: int
        :param since:
        :type since: datetime.datetime
        :param tag_id: id of a tag
        :type tag_id: int
        :param title:
        :type title: str
        :return: list of person objects
        :rtype: list
        """
        params = {}
        if page:
            params['n'] = int(page) * cls.OFFSET
        if since:
            params['since'] = since.strftime(cls.COLLECTION_DATETIME)
        if tag_id:
            params['tag_id'] = str(tag_id)
        if title:
            params['title'] = title

        return super().list(params)



from highton import call_mixins
from highton.models import HightonModel
from highton.highton_constants import HightonConstants
from highton import fields


class Case(
    HightonModel,
    call_mixins.CreateCallMixin,
    call_mixins.UpdateCallMixin,
    call_mixins.DetailCallMixin,
    call_mixins.ListCallMixin,
    call_mixins.DeleteCallMixin,
    call_mixins.ListNoteCallMixin,
    call_mixins.ListEmailCallMixin,
    call_mixins.ListTagCallMixin,
    call_mixins.CreateTagCallMixin,
    call_mixins.DeleteTagCallMixin,
    call_mixins.ListTaskCallMixin,
    call_mixins.CreateNoteCallMixin,
):
    """

    :ivar id: fields.IntegerField(name=HightonConstants.ID)
    :ivar author_id: fields.IntegerField(name=HightonConstants.AUTHOR_ID)
    :ivar closed_at: fields.DatetimeField(name=HightonConstants.CLOSED_AT)
    :ivar created_at: fields.DatetimeField(name=HightonConstants.CREATED_AT)
    :ivar updated_at: fields.DatetimeField(name=HightonConstants.UPDATED_AT)
    :ivar name: fields.StringField(name=HightonConstants.NAME)
    :ivar visible_to: fields.StringField(name=HightonConstants.VISIBLE_TO)
    :ivar group_id: fields.IntegerField(name=HightonConstants.GROUP_ID)
    :ivar owner_id: fields.IntegerField(name=HightonConstants.OWNER_ID)
    :ivar background: fields.StringField(name=HightonConstants.BACKGROUND)
    :ivar parties: fields.ListField(name=HightonConstants.PARTIES, init_class=Party)
    :ivar associated_parties: fields.ListField(name=HightonConstants.ASSOCIATED_PARTIES, init_class=AssociatedParty)
    """

    ENDPOINT = HightonConstants.CASES
    TAG_NAME = HightonConstants.CASE

    def __init__(self, **kwargs):
        from highton.models import (
            Party,
            AssociatedParty,
        )

        self.author_id = fields.IntegerField(name=HightonConstants.AUTHOR_ID)
        self.closed_at = fields.DatetimeField(name=HightonConstants.CLOSED_AT)
        self.created_at = fields.DatetimeField(name=HightonConstants.CREATED_AT)
        self.updated_at = fields.DatetimeField(name=HightonConstants.UPDATED_AT)
        self.name = fields.StringField(name=HightonConstants.NAME)
        self.visible_to = fields.StringField(name=HightonConstants.VISIBLE_TO)
        self.group_id = fields.IntegerField(name=HightonConstants.GROUP_ID)
        self.owner_id = fields.IntegerField(name=HightonConstants.OWNER_ID)
        self.background = fields.StringField(name=HightonConstants.BACKGROUND)
        self.parties = fields.ListField(name=HightonConstants.PARTIES, init_class=Party)
        self.associated_parties = fields.ListField(name=HightonConstants.ASSOCIATED_PARTIES, init_class=AssociatedParty)

        super().__init__(**kwargs)



from highton.models import HightonModel
from highton.highton_constants import HightonConstants
from highton import fields


class Status(
    HightonModel,
):
    """

    :ivar id: fields.IntegerField(name=HightonConstants.ID)
    :ivar name: fields.StringField(name=HightonConstants.NAME)
    """
    TAG_NAME = HightonConstants.STATUS

    def __init__(self, **kwargs):
        self.name = fields.StringField(name=HightonConstants.NAME)
        super().__init__(**kwargs)



from highton.models import HightonModel
from highton.highton_constants import HightonConstants
from highton import (
    fields,
    call_mixins,
)


class SubjectField(
    HightonModel,
    call_mixins.ListCallMixin,
    call_mixins.CreateCallMixin,
    call_mixins.UpdateCallMixin,
    call_mixins.DeleteCallMixin,
):
    """
    :ivar id: fields.IntegerField(name=HightonConstants.ID)
    :ivar label: fields.StringField(name=HightonConstants.LABEL)
    :ivar type: fields.StringField(name=HightonConstants.TYPE)
    """
    TAG_NAME = HightonConstants.SUBJECT_FIELD
    ENDPOINT = HightonConstants.SUBJECT_FIELDS

    def __init__(self, **kwargs):
        self.label = fields.StringField(name=HightonConstants.LABEL)
        self.type = fields.StringField(name=HightonConstants.TYPE)

        super().__init__(**kwargs)

    @classmethod
    def list(cls, params=None):
        """
        Returns all subject fields
        params should have a type field with (party, deal)

        :param params:
        :type params: dict
        :return:
        :rtype:
        """
        return super().list(params)



import datetime
from xml.etree import ElementTree

from highton.fields.field import Field
from highton.fields.field_constants import FieldConstants


class BooleanField(Field):
    """
    The BooleanField represents a field which value will be parsed to Highrise specific boolean format

    """

    TRUE = 'true'
    FALSE = 'false'

    MAPPING = {
        True: TRUE,
        False: FALSE,
    }

    def encode(self):
        """

        :return:
        :rtype: xml.etree.ElementTree.Element
        """
        element = ElementTree.Element(
            self.name,
            attrib={'type': FieldConstants.DATE},
        )
        element.text = self.MAPPING[self.value]
        return element

    def decode(self, element):
        """

        :param element: the element which needs to be parsed
        :type element: xml.etree.ElementTree.Element
        :return: datetime.date
        :rtype: the parsed date object
        """
        return element.text == self.TRUE



import datetime
from xml.etree import ElementTree

from highton.fields.field import Field
from highton.fields.field_constants import FieldConstants


class DatetimeField(Field):
    DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%SZ'

    def encode(self):
        """

        :return: the encoded element
        :rtype: xml.etree.ElementTree.Element
        """
        element = ElementTree.Element(
            self.name,
            attrib={
                'type': FieldConstants.DATETIME
            },
        )
        element = self._set_nil(element, lambda value: value.strftime(self.DATETIME_FORMAT))
        return element

    def decode(self, element):
        """

        :param element: the element which needs to be parsed
        :type element: xml.etree.ElementTree.Element
        :return: the parsed datetime object
        :rtype: datetime.datetime
        """
        if element.text:
            return datetime.datetime.strptime(element.text, self.DATETIME_FORMAT)
        else:
            return None



from highton.models import HightonModel
from highton.highton_constants import HightonConstants
from highton import fields


class EmailAddress(
    HightonModel,
):
    """
    :ivar id: fields.IntegerField(name=HightonConstants.ID)
    :ivar location: fields.StringField(name=HightonConstants.LOCATION)
    :ivar address: fields.StringField(name=HightonConstants.ADDRESS)
    """
    TAG_NAME = HightonConstants.EMAIL_ADDRESS

    def __init__(self, **kwargs):
        self.location = fields.StringField(name=HightonConstants.LOCATION, required=True)
        self.address = fields.StringField(name=HightonConstants.ADDRESS, required=True)

        super().__init__(**kwargs)



from highton import call_mixins
from highton.models import HightonModel
from highton.highton_constants import HightonConstants
from highton import fields


class User(
    HightonModel,
    call_mixins.ListCallMixin,
    call_mixins.DetailCallMixin,
):
    """
    :ivar id: fields.IntegerField(name=HightonConstants.ID)
    :ivar name: fields.StringField(name=HightonConstants.NAME)
    :ivar email_address: fields.StringField(name=HightonConstants.EMAIL_ADDRESS)
    :ivar admin: fields.BooleanField(name=HightonConstants.ADMIN)
    :ivar token: fields.StringField(name=HightonConstants.TOKEN)
    :ivar dropbox: fields.StringField(name=HightonConstants.DROPBOX)
    :ivar created_at: fields.DatetimeField(name=HightonConstants.CREATED_AT)
    :ivar updated_at: fields.DatetimeField(name=HightonConstants.UPDATED_AT)
    """
    ENDPOINT = HightonConstants.USERS
    TAG_NAME = HightonConstants.USER

    def __init__(self, **kwargs):
        self.name = fields.StringField(name=HightonConstants.NAME)
        self.email_address = fields.StringField(name=HightonConstants.EMAIL_ADDRESS)
        self.admin = fields.BooleanField(name=HightonConstants.ADMIN)
        self.token = fields.StringField(name=HightonConstants.TOKEN)
        self.dropbox = fields.StringField(name=HightonConstants.DROPBOX)
        self.created_at = fields.DatetimeField(name=HightonConstants.CREATED_AT)
        self.updated_at = fields.DatetimeField(name=HightonConstants.UPDATED_AT)

        super().__init__(**kwargs)

    @classmethod
    def me(cls):
        """
        Returns information about the currently authenticated user.

        :return:
        :rtype: User
        """
        return fields.ObjectField(name=cls.ENDPOINT, init_class=cls).decode(
            cls.element_from_string(
                cls._get_request(endpoint=cls.ENDPOINT + '/me').text
            )
        )



from highton import call_mixins
from highton.models import HightonModel
from highton.highton_constants import HightonConstants
from highton import fields


class Tag(
    HightonModel,
    call_mixins.ListCallMixin,
):
    """
    :ivar id: fields.IntegerField(name=HightonConstants.ID)
    :ivar name: fields.StringField(name=HightonConstants.NAME)
    """
    ENDPOINT = HightonConstants.TAGS
    TAG_NAME = HightonConstants.TAG

    def __init__(self, **kwargs):
        self.name = fields.StringField(name=HightonConstants.NAME)

        super().__init__(**kwargs)



from highton.models import HightonModel
from highton.highton_constants import HightonConstants
from highton import (
    fields,
    call_mixins,
)
from highton.models.attachment import Attachment


class Email(
    HightonModel,
    call_mixins.DetailCallMixin,
    call_mixins.CreateCallMixin,
    call_mixins.UpdateCallMixin,
    call_mixins.DeleteCallMixin,
    call_mixins.ListCommentCallMixin,
):
    """

    :ivar id: fields.IntegerField(name=HightonConstants.ID)
    :ivar title: fields.StringField(name=HightonConstants.TITLE, required=True)
    :ivar body: fields.StringField(name=HightonConstants.BODY, required=True)
    :ivar subject_id: fields.IntegerField(name=HightonConstants.SUBJECT_ID, required=True)
    :ivar subject_type: fields.StringField(name=HightonConstants.SUBJECT_TYPE, required=True)
    :ivar subject_name: fields.StringField(name=HightonConstants.SUBJECT_NAME)
    :ivar author_id: fields.IntegerField(name=HightonConstants.AUTHOR_ID)
    :ivar collection_id: fields.IntegerField(name=HightonConstants.COLLECTION_ID)
    :ivar collection_type: fields.StringField(name=HightonConstants.COLLECTION_TYPE)
    :ivar visible_to: fields.StringField(name=HightonConstants.VISIBLE_TO)
    :ivar owner_id: fields.IntegerField(name=HightonConstants.OWNER_ID)
    :ivar group_id: fields.IntegerField(name=HightonConstants.GROUP_ID)
    :ivar updated_at: fields.DatetimeField(name=HightonConstants.UPDATED_AT)
    :ivar created_at: fields.DatetimeField(name=HightonConstants.CREATED_AT)
    :ivar attachments: fields.ListField(name=HightonConstants.ATTACHMENTS, init_class=Attachment)
    """
    TAG_NAME = HightonConstants.EMAIL
    ENDPOINT = HightonConstants.EMAILS

    def __init__(self, **kwargs):
        self.title = fields.StringField(name=HightonConstants.TITLE, required=True)
        self.body = fields.StringField(name=HightonConstants.BODY, required=True)
        self.subject_id = fields.IntegerField(name=HightonConstants.SUBJECT_ID, required=True)
        self.subject_type = fields.StringField(name=HightonConstants.SUBJECT_TYPE, required=True)
        self.subject_name = fields.StringField(name=HightonConstants.SUBJECT_NAME)
        self.author_id = fields.IntegerField(name=HightonConstants.AUTHOR_ID)
        self.collection_id = fields.IntegerField(name=HightonConstants.COLLECTION_ID)
        self.collection_type = fields.StringField(name=HightonConstants.COLLECTION_TYPE)
        self.visible_to = fields.StringField(name=HightonConstants.VISIBLE_TO)
        self.owner_id = fields.IntegerField(name=HightonConstants.OWNER_ID)
        self.group_id = fields.IntegerField(name=HightonConstants.GROUP_ID)
        self.updated_at = fields.DatetimeField(name=HightonConstants.UPDATED_AT)
        self.created_at = fields.DatetimeField(name=HightonConstants.CREATED_AT)
        self.attachments = fields.ListField(name=HightonConstants.ATTACHMENTS, init_class=Attachment)

        super().__init__(**kwargs)



from abc import ABCMeta
from xml.etree import ElementTree

from highton import fields


class FieldDoesNotExist(Exception):
    def __init__(self, obj, element):
        self.obj = obj
        self.element = element

    def __str__(self):
        return 'The field "{tag}" does not exist in {object}'.format(tag=self.element.tag, object=self.obj)


class XMLDecoder(metaclass=ABCMeta):
    """
    This class is an abstract class which helps to decode a XML String to the actual object which inherits from it

    """

    def __getattribute__(self, name):
        attribute_value = object.__getattribute__(self, name)
        if isinstance(attribute_value, fields.Field):
            return attribute_value.value
        return attribute_value

    def _get_field_names_to_attributes(self):

        return {
            value.name: key
            for key, value in self.__dict__.items()
            if isinstance(value, fields.Field)
        }

    def _get_field(self, attribute_name):
        return self.__dict__[attribute_name]

    @staticmethod
    def _set_field(xml_decoder_object, field_names_to_attributes, child_element):
        try:
            field = xml_decoder_object._get_field(
                field_names_to_attributes[child_element.tag]
            )
            field.value = field.decode(child_element)
        except KeyError:
            raise FieldDoesNotExist(xml_decoder_object, child_element)

    @staticmethod
    def element_from_string(string):
        """
        Make a Element from a str

        :param string: string you want to parse
        :type string: str
        :return: the parsed xml string
        :rtype: xml.etree.ElementTree.Element
        """
        return ElementTree.fromstring(string)

    @classmethod
    def decode(cls, root_element):
        """
        Decode the object to the object

        :param root_element: the parsed xml Element
        :type root_element: xml.etree.ElementTree.Element
        :return: the decoded Element as object
        :rtype: object
        """
        new_object = cls()
        field_names_to_attributes = new_object._get_field_names_to_attributes()
        for child_element in root_element:
            new_object._set_field(new_object, field_names_to_attributes, child_element)
        return new_object



from .call import Call
from .list_call_mixin import ListCallMixin
from .detail_call_mixin import DetailCallMixin
from .update_call_mixin import UpdateCallMixin
from .delete_call_mixin import DeleteCallMixin
from .create_call_mixin import CreateCallMixin
from .list_note_call_mixin import ListNoteCallMixin
from .list_email_call_mixin import ListEmailCallMixin
from .list_comment_call_mixin import ListCommentCallMixin
from .list_tag_call_mixin import ListTagCallMixin
from .create_tag_call_mixin import CreateTagCallMixin
from .delete_tag_call_mixin import DeleteTagCallMixin
from .list_task_call_mixin import ListTaskCallMixin
from .create_note_call_mixin import CreateNoteCallMixin
from .search_call_mixin import SearchCallMixin



from highton.call_mixins import Call


class CreateNoteCallMixin(Call):
    """
    A mixin to create a Note to inherited class
    These could be: people || companies || kases || deals

    """

    def add_note(self, body):
        """
        Create a Note to current object

        :param body: the body of the note
        :type body: str
        :return: newly created Note
        :rtype: Tag
        """
        from highton.models.note import Note
        created_id = self._post_request(
            endpoint=self.ENDPOINT + '/' + str(self.id) + '/' + Note.ENDPOINT,
            data=self.element_to_string(
                Note(body=body).encode()
            )
        ).headers.get('Location').replace('.xml', '').split('/')[-1]
        return Note.get(created_id)



from abc import ABCMeta
from xml.etree import ElementTree


class Field(metaclass=ABCMeta):
    """
    This is an abstract class for a field, which makes the base for an encode and decode method

    :ivar name: name of the field which needed to serialize this field to a xml object
    :ivar required: if the field is required when you create or update an object
    :ivar value: the value of the field
    """
    def __init__(self, name, required=False):
        self.name = name
        self.required = required
        self.value = None

    def encode(self):
        """
        Encodes the value of the field and put it in the element
        also make the check for nil=true if there is one

        :return: returns the encoded element
        :rtype: xml.etree.ElementTree.Element
        """
        element = ElementTree.Element(self.name)
        element = self._set_nil(element, lambda value: str(value))
        return element

    def decode(self, element):
        """
        Decodes the value of the element

        :param element:
        :type element:
        :return:
        :rtype:
        """
        return element.text

    def _set_nil(self, element, value_parser):
        """
        Method to set an attribute of the element.
        If the value of the field is None then set the nil='true' attribute in the element

        :param element: the element which needs to be modified
        :type element: xml.etree.ElementTree.Element
        :param value_parser: the lambda function which changes will be done to the self.value
        :type value_parser: def
        :return: the element with or without the specific attribute
        :rtype: xml.etree.ElementTree.Element
        """
        if self.value:
            element.text = value_parser(self.value)
        else:
            element.attrib['nil'] = 'true'
        return element



from .field import Field
from .date_field import DateField
from .datetime_field import DatetimeField
from .integer_field import IntegerField
from .string_field import StringField
from .list_field import ListField
from .object_field import ObjectField
from .boolean_field import BooleanField


from highton.models import HightonModel
from highton.highton_constants import HightonConstants
from highton import fields


class SubjectData(
    HightonModel,
):
    """
    :ivar id: fields.IntegerField(name=HightonConstants.ID)
    :ivar value: fields.StringField(name=HightonConstants.VALUE)
    :ivar type: fields.StringField(name=HightonConstants.TYPE)
    :ivar subject_field_id: fields.StringField(name=HightonConstants.SUBJECT_FIELD_ID)
    :ivar subject_field_label: fields.StringField(name=HightonConstants.SUBJECT_FIELD_LABEL)
    """
    TAG_NAME = HightonConstants.SUBJECT_DATA

    def __init__(self, **kwargs):
        self.value = fields.StringField(name=HightonConstants.VALUE)
        self.type = fields.StringField(name=HightonConstants.TYPE)
        self.subject_field_id = fields.StringField(name=HightonConstants.SUBJECT_FIELD_ID)
        self.subject_field_label = fields.StringField(name=HightonConstants.SUBJECT_FIELD_LABEL)

        super().__init__(**kwargs)



from setuptools import setup

setup(
    name='Highton',
    version='2.0.1',
    license='Apache License 2.0',
    description='A Python library for Highrise',
    long_description='A beautiful Python - Highrise - Wrapper.',
    url='https://github.com/seibert-media/Highton',
    author='Julia Giebelhausen, Jakob Löhnertz, Michael Bykovski',
    author_email='brogrammers@seibert-media.net',
    packages=[
        'highton',
        'highton.call_mixins',
        'highton.fields',
        'highton.models',
        'highton.parsing',
    ],
    install_requires=[
        'requests'
    ],
    keywords='seibertmedia seibert media python api wrapper highrise highton',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
    ]
)



from highton.models import HightonModel
from highton.highton_constants import HightonConstants
from highton import fields


class Address(
    HightonModel,
):
    """

    :ivar id: fields.IntegerField(name=HightonConstants.ID)
    :ivar location: fields.StringField(name=HightonConstants.LOCATION)
    :ivar city: fields.StringField(name=HightonConstants.CITY)
    :ivar country: fields.StringField(name=HightonConstants.COUNTRY)
    :ivar state: fields.StringField(name=HightonConstants.STATE)
    :ivar street: fields.StringField(name=HightonConstants.STREET)
    :ivar zip: fields.StringField(name=HightonConstants.ZIP)
    """
    TAG_NAME = HightonConstants.ADDRESS

    def __init__(self, **kwargs):
        self.location = fields.StringField(name=HightonConstants.LOCATION)
        self.city = fields.StringField(name=HightonConstants.CITY)
        self.country = fields.StringField(name=HightonConstants.COUNTRY)
        self.state = fields.StringField(name=HightonConstants.STATE)
        self.street = fields.StringField(name=HightonConstants.STREET)
        self.zip = fields.StringField(name=HightonConstants.ZIP)

        super().__init__(**kwargs)



from highton.models import HightonModel
from highton.highton_constants import HightonConstants
from highton import fields


class Attachment(
    HightonModel,
):
    """

    :ivar id: fields.IntegerField(name=HightonConstants.ID)
    :ivar url: fields.StringField(name=HightonConstants.URL)
    :ivar name: fields.StringField(name=HightonConstants.NAME)
    :ivar size: fields.IntegerField(name=HightonConstants.SIZE)

    """
    TAG_NAME = HightonConstants.ATTACHMENT

    def __init__(self, **kwargs):
        self.url = fields.StringField(name=HightonConstants.URL)
        self.name = fields.StringField(name=HightonConstants.NAME)
        self.size = fields.IntegerField(name=HightonConstants.SIZE)

        super().__init__(**kwargs)
